from osv import fields,osv
from tools.translate import _


class purchase_order(osv.osv):
    _name = "tr_purchase.order"
    _description = "Purchase Order"

    def _calc_total(self, cr, uid, ids, field_name, arg, context):
        print "----_calc_total"
        company_obj = self.pool.get('res.company')
        
        res = {}
        for order in self.browse(cr, uid, ids, context=context):
            # Example of getting another record e.g. the company
            company_id = order.supplier_id.company_id.id
            print company_obj.browse(cr, uid, company_id, context=context).currency_id.name

            total = 0.0
            for line in order.line_ids:
                total += line.amount
            res[order.id] = total
        print res

        return res


    def _check_permissions(self, cr, uid, ids, field_name, arg, context):

        res = {}

        for i in ids:
            if not i:
                continue

            # Get the HR Officer/Manager id's
            group_obj = self.pool.get('res.groups')
            manager_ids = group_obj.search(cr, uid, ['|',('name','=', 'Human Resources / HR Officer'),('name','=', 'Human Resources / HR Manager')])
             
            # Get the user and see what groups he/she is in
            user_obj = self.pool.get('res.users')
            user =  user_obj.browse(cr, uid, uid, context=context)
            group_ids = []
            for grp in user.groups_id:
                group_ids.append(grp.id)
             
            if (manager_ids[0] in group_ids) or (manager_ids[1] in group_ids):
                res[i] ='Manager'
            else:
                res[i] = 'User'

        print res
        return res


    _columns= {
        'name': fields.char('Description', size=64, required=True, help=_("A description of the purchase order")),
        'state': fields.selection([('draft','Draft'), ('approved','Approved'), ('waiting','Waiting Approval'), ('not_approved','Not Approved')], 'State', required=True, readonly=True),
        'note': fields.text('Note'),
        'user_id': fields.many2one('res.users', 'User', required=True),
        'supplier_id': fields.many2one('res.partner', 'Supplier', domain=[('supplier','=',True)], required=True),
        'line_ids': fields.one2many('tr_purchase.order.line', 'order_id', 'Items'),
        'order_total': fields.function(_calc_total, type='float', method=True,
                                       string="Order Total", store=True),
        'website': fields.related('supplier_id', 'website', type="char",
                                   relation="res.partner", string='Website', readonly=True),
        'department_id': fields.many2one('hr.department', 'Department'),
        'date': fields.date('Order Date'),
        'permissions': fields.function(_check_permissions, type='char', method=True,
                                       string="Permissions"),
    }

    _defaults = {
        'state': 'draft',
        'user_id': lambda self, cr, uid, context=None: uid,
        }


    def onchange_user(self, cr, uid, ids, user_id):
        """
        Limit the tasks to the ones in this project.
        """
        print "-----onchange_user"
        print "user_id:", user_id
        
        res = {}
        if user_id:
            # Look up the department from the user id
            department = False
            empl_obj = self.pool.get('hr.employee')
            empl_ids = empl_obj.search(cr, uid, [('user_id','=',user_id)])
            if len(empl_ids)>0:
                employee = empl_obj.browse(cr, uid, empl_ids[0])
                department = employee.department_id.id
                print "empl_ids:",empl_ids ,"   employee:", employee,"   department_id",employee.department_id,"  department:", department

            res["value"] = {"department_id": department}
            #res["domain"] = {"task_id":"[('project_id','=',%d)]" % project_id}
            res['warning'] = {'title':'User has been changed', 'message': 'Please check that the correct department has been defined'}

        return res


    def create(self, cr, uid, vals, context=None):
        """
        Validate the record on creation.
        """
        self.record_validate(cr, uid, [], vals, context)
        result = super(purchase_order,self).create(cr, uid, vals, context)
        return result


    def write(self, cr, uid, ids, vals, context=None):
        """
        Validate the record when updating it.
        """
        result = self.record_validate(cr, uid, ids, vals, context)
        super(purchase_order, self).write(cr, uid, ids, vals, context)

        return True


    def record_validate(self, cr, uid, ids, vals, context):
        print "----record_validate:", vals
        if vals.get('supplier_id'):
            supp_obj = self.pool.get("res.partner")
            supplier = supp_obj.browse(cr, uid, vals.get('supplier_id'), context=context)
            if not supplier.user_id:
                raise osv.except_osv(_('UserError'),"The supplier '%s' does not have a Salesman" % supplier.name)


purchase_order()


class purchase_order_line(osv.osv):
    _name = "tr_purchase.order.line"
    _description = "Purchase Order Line"

    def _calc_amount(self, cr, uid, ids, field_name, arg, context):
        print "----------calc_amount"
        print "ids:", ids, "   field_name", field_name
        print "context", context

        res = {}

        for line in self.browse(cr, uid, ids, context=context):
            res[line.id] = line.qty * line.unit_price

        print res
        return res


    _columns = {
        'name': fields.char('Name', size=64, required=True),
        'order_id': fields.many2one('tr_purchase.order', 'Purchase Order', required=True),
        'qty': fields.integer('Quantity', required=True),
        'unit_price': fields.float('Unit Price', required=True),
        'amount': fields.function(_calc_amount, type="float", 
                                  method=True, string="Amount", store=True),
        }

    _defaults = {
        'qty': 1,
        }

purchase_order_line()


